/*
 * MacFace AvP[VNX
 *
 * $Id$
 * 
 * project created on 2004/06/02 at 2:43
 * 
 */
using System;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Data;
using System.IO;
using System.Reflection;

namespace MacFace.FloatApp
{
	/// <summary>
	/// MacFaceApp ̊Tv̐łB
	/// </summary>
	public class MacFaceApp : ApplicationContext
	{

		private Configuration config;

		private NotifyIcon notifyIcon;
		private PatternWindow patternWindow;
		
		private System.Windows.Forms.Timer updateTimer;
		private CPUUsageCounter cpuCounter;
		private MemoryUsageCounter memoryCounter;

		private Form statusWindow;

		private CPUUsage[] cpuHistory;
		private int cpuHistoryHead;
		private int cpuHistoryCount;

		private MemoryUsage[] memHistory;
		private int memHistoryHead;
		private int memHistoryCount;

		[STAThread]
		public static void Main(string[] args)
		{
			MacFaceApp app = new MacFaceApp();
			app.StartApplication();
		}

		public MacFaceApp()
		{
			config = Configuration.GetInstance();
			config.Load();

			cpuHistory = new CPUUsage[60];
			cpuHistoryCount = 0;
			cpuHistoryHead = 0;

			memHistory = new MemoryUsage[60];
			memHistoryCount = 0;
			memHistoryHead = 0;

			cpuCounter = new CPUUsageCounter();
			memoryCounter = new MemoryUsageCounter();

			updateTimer = new System.Windows.Forms.Timer();
			updateTimer.Enabled = false;
			updateTimer.Interval = 1000;
			updateTimer.Tick += new EventHandler(this.CountProcessorUsage);

			InitializeComponent();
		}

		void InitializeComponent() 
		{
			// ReLXgj[
			ContextMenu contextMenu = new System.Windows.Forms.ContextMenu();
			MenuItem menuItemPatternSelect = new System.Windows.Forms.MenuItem();
			MenuItem menuItemConfigure = new System.Windows.Forms.MenuItem();
			MenuItem menuItemExit = new System.Windows.Forms.MenuItem();

			menuItemPatternSelect.Text = "p^[̑I(&S)";
			menuItemPatternSelect.Click += new System.EventHandler(menuItemPatternSelect_Click);

			menuItemConfigure.Text = "MacFace ̐ݒ(&C)...";
			menuItemConfigure.Click +=new EventHandler(menuItemConfigure_Click);

			menuItemExit.Index = 0;
			menuItemExit.Text = "I(&X)";
			menuItemExit.Click += new System.EventHandler(menuItemExit_Click);

			contextMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
					menuItemPatternSelect, menuItemConfigure, new MenuItem("-"), menuItemExit});

			// ʒmACR
			Assembly asm = Assembly.GetExecutingAssembly();
			this.notifyIcon = new System.Windows.Forms.NotifyIcon();
			this.notifyIcon.Text = "MacFace";
			this.notifyIcon.Icon = new Icon(asm.GetManifestResourceStream("MacFace.FloatApp.App.ico"));
			this.notifyIcon.Visible = true;
			this.notifyIcon.ContextMenu = contextMenu;

			// p^[EChE
			this.patternWindow = new PatternWindow();

			// Xe[^XEChE		
			statusWindow = new Form();
			statusWindow.ClientSize = new System.Drawing.Size(310, 215);
			statusWindow.FormBorderStyle = System.Windows.Forms.FormBorderStyle.FixedToolWindow;
			statusWindow.ControlBox = false;
			statusWindow.Icon = new Icon(asm.GetManifestResourceStream("MacFace.FloatApp.App.ico"));
			statusWindow.Text = "Xe[^X";
			statusWindow.Paint +=new PaintEventHandler(statusWindow_Paint);
		}

		public void StartApplication()
		{
			// p^[ǂݍ
			bool result = false;
			if (Directory.Exists(config.FaceDefPath))
			{
				result = LoadFaceDefine(config.FaceDefPath);
			}

			if (!result)
			{
				if (!SelectFaceDefine(Application.StartupPath))
				{
					Application.Exit();
					return;
				}
			}

			statusWindow.Show();

			patternWindow.Location = config.Location;
			patternWindow.Opacity = (float)config.Opacity / 100;
			patternWindow.PatternSize = (float)config.PatternSize / 100;
			patternWindow.TransparentMouseMessage = config.TransparentMouseMessage;
			patternWindow.Refresh();

			patternWindow.Show();
			updateTimer.Start();

			Application.ApplicationExit += new EventHandler(Application_ApplicationExit);
			Application.Run(this);
		}

		void Application_ApplicationExit(object sender, EventArgs e)
		{
			notifyIcon.Visible = false;

			// ۑ
			config.Location = patternWindow.Location;
			config.Save();
		}

		/*
		 * p^[`tH_I
		 */
		public bool SelectFaceDefine()
		{
			return SelectFaceDefine(Application.StartupPath);
		}

		public bool SelectFaceDefine(string defaultPath)
		{
			while (true) 
			{
				FolderBrowserDialog folderBrowser = new FolderBrowserDialog();
				folderBrowser.SelectedPath = defaultPath;
				folderBrowser.Description = "p^[t@C݂̑tH_IĂB";
				if (folderBrowser.ShowDialog() == DialogResult.OK) 
				{
					if (LoadFaceDefine(folderBrowser.SelectedPath)) 
					{
						return true;
					}
				}
				else 
				{
					return false;
				}
			}

		}


		public bool LoadFaceDefine(string path)
		{
			FaceDef newFaceDef = null;
			string plistPath = Path.Combine(path, "faceDef.plist");

			if (!File.Exists(plistPath))
			{
				System.Windows.Forms.MessageBox.Show(
					String.Format("w肳ꂽtH_Ɋp^[`XMLt@C \"faceDef.plist\" ݂܂B\n\ntH_:\n{0}", path),
					"MacFace for Windows", MessageBoxButtons.OK, MessageBoxIcon.Error);
				return false;
			}
			
			try 
			{
				newFaceDef = new MacFace.FaceDef(path);
			} 
			catch (System.IO.IOException ie) 
			{
				System.Windows.Forms.MessageBox.Show(
					String.Format("p^[`XMLt@CǂݍލۂɃG[܂B\n\n:\n{0}",
					ie.ToString()), "MacFace for Windows", MessageBoxButtons.OK, MessageBoxIcon.Error);
				
				return false;
			}
			catch (System.Xml.XmlException xe) 
			{
				System.Windows.Forms.MessageBox.Show(
					String.Format("p^[`XMLt@CǍݒɃG[܂B\n\n:\n{0}",
					xe.ToString()), "MacFace for Windows", MessageBoxButtons.OK, MessageBoxIcon.Error);
				
				return false;
			}

			// p^[ւ͍XV~߂Ă
			if (updateTimer != null) updateTimer.Stop();

			patternWindow.FaceDef = newFaceDef;
			patternWindow.Refresh();

			notifyIcon.Text = "MacFace - " + patternWindow.FaceDef.Title;

			// XVĊJ
			if (updateTimer != null) updateTimer.Start();

			return true;
		}

		public void CountProcessorUsage(object sender, EventArgs e)
		{
			CPUUsage cpuUsage = cpuCounter.CurrentUsage();
			MemoryUsage memUsage = memoryCounter.CurrentUsage();

			cpuHistory[cpuHistoryHead++] = cpuUsage;
			if (cpuHistoryHead >= cpuHistory.Length) cpuHistoryHead = 0;
			if (cpuHistoryCount < cpuHistory.Length) cpuHistoryCount++;

			memHistory[memHistoryHead++] = memUsage;
			if (memHistoryHead >= memHistory.Length) memHistoryHead = 0;
			if (memHistoryCount < memHistory.Length) memHistoryCount++;

			int pattern = cpuUsage.Active / 10;

			FaceDef.PatternSuite suite = FaceDef.PatternSuite.Normal;

			int avilable = (int)MemoryUsageCounter.TotalVisibleMemorySize * 1024 - memUsage.Committed;
			if (avilable < (10 * 1024 *1024)) 
			{
				suite = FaceDef.PatternSuite.MemoryInsufficient;
			} 
			else if (memUsage.Available < (30 * 1024 *1024)) 
			{
				suite = FaceDef.PatternSuite.MemoryDecline;
			}

			int markers = FaceDef.MarkerNone;
			if (memUsage.Pagein > 0) markers += FaceDef.MarkerPageIn;
			if (memUsage.Pageout > 0) markers += FaceDef.MarkerPageOut;

			patternWindow.UpdatePattern(suite, pattern, markers);

			statusWindow.Invalidate();
		}

		/*
		 * j[NbNCxg
		 */
		public void menuItemPatternSelect_Click(object sender, System.EventArgs e)
		{
			SelectFaceDefine(patternWindow.FaceDef.Path);	
		}

		public void menuItemExit_Click(object sender, System.EventArgs e)
		{
			updateTimer.Stop();
			patternWindow.Close();
			ExitThread();
		}

		private void menuItemConfigure_Click(object sender, EventArgs e)
		{
			ConfigurationForm configForm = new ConfigurationForm();
			configForm.ConfigChanged += new ConfigChangedEvent(configForm_ConfigChanged);
			configForm.Show();
		}

		private void configForm_ConfigChanged()
		{
			if (patternWindow.FaceDef.Path != config.FaceDefPath) 
			{
				bool result = LoadFaceDefine(config.FaceDefPath);
				// p^[ύXɎsݒɖ߂
				if (!result) 
				{
					config.FaceDefPath = patternWindow.FaceDef.Path;
				}
			}
			
			patternWindow.Opacity = (float)config.Opacity / 100;
			patternWindow.PatternSize = (float)config.PatternSize / 100;
			patternWindow.TransparentMouseMessage = config.TransparentMouseMessage;
			patternWindow.Refresh();
		}

		private void statusWindow_Paint(object sender, PaintEventArgs e)
		{
			Graphics g = e.Graphics;
			g.SmoothingMode = SmoothingMode.AntiAlias;

			g.FillRectangle(new SolidBrush(Color.White), 5, 5, 300, 100);
			Pen pen = new Pen(Color.FromArgb(220, 220, 220), 1F);
			for (int y = 5; y < 105; y += 10) 
			{
				g.DrawLine(pen, 5, y, 305, y);
			}
			g.DrawLine(Pens.Gray, 5, 55, 305, 55);

			g.DrawRectangle(Pens.Black, 4, 4, 301, 101);

			g.FillRectangle(new SolidBrush(Color.White), 5, 110, 300, 100);
			for (int y = 210; y > 110; y -= 7) 
			{
				g.DrawLine(pen, 5, y, 305, y);
			}
			g.DrawLine(Pens.Gray, 5, 210-5*7, 305, 210-5*7);
			g.DrawRectangle(Pens.Black, 4, 109, 301, 101);


			if (cpuHistoryCount >= 2) 
			{
				Point[] userGraph = new Point[cpuHistoryCount+2];
				Point[] sysGraph = new Point[cpuHistoryCount+2];

				userGraph[cpuHistoryCount+0].X = 305 - cpuHistoryCount * 5;
				userGraph[cpuHistoryCount+0].Y = 105 - 0;
				userGraph[cpuHistoryCount+1].X = 305 - 0 * 5;
				userGraph[cpuHistoryCount+1].Y = 105 - 0;

				sysGraph[cpuHistoryCount+0].X = 305 - cpuHistoryCount * 5;
				sysGraph[cpuHistoryCount+0].Y = 105 - 0;
				sysGraph[cpuHistoryCount+1].X = 305 - 0 * 5;
				sysGraph[cpuHistoryCount+1].Y = 105 - 0;

				int pos = cpuHistoryHead - 1;
				for (int i = 0; i < cpuHistoryCount; i++) 
				{
					if (pos < 0) pos = cpuHistory.Length - 1;
					CPUUsage usage = cpuHistory[pos];
					userGraph[i].X = sysGraph[i].X = 305 - i * 5;
					userGraph[i].Y = 105 - usage.Active;
					sysGraph[i].Y = 105 - usage.System;
					pos--;
				}

				g.FillPolygon(new SolidBrush(Color.FromArgb(50, 0, 0, 255)), userGraph);
				g.DrawPolygon(new Pen(Color.FromArgb(0, 0, 255), 1F), userGraph);
				g.FillPolygon(new SolidBrush(Color.FromArgb(50, 255, 0, 0)), sysGraph);
			}

			int totalMemory = (int)MemoryUsageCounter.TotalVisibleMemorySize * 1024;
			double rate = 70.0 / totalMemory;

			g.SmoothingMode = SmoothingMode.None;
			Brush commitedBrush = new SolidBrush(Color.FromArgb(180, 255, 145, 0));
			Brush uncommitedBrush = new SolidBrush(Color.FromArgb(180, 180, 200, 255));
			Brush availableBrush = new SolidBrush(Color.FromArgb(180, 100, 100, 255));

			int posu = memHistoryHead - 1;
			for (int i = 0; i < memHistoryCount; i++) 
			{
				if (posu < 0) posu = memHistory.Length - 1;
				MemoryUsage usage = memHistory[posu];

				int x, y, w, h;

				x = 305 - i * 5 - 5;
				w = 5;

				h = (int)(usage.Committed * rate);
				y = 210 - h;
				g.FillRectangle(commitedBrush, x, y, w, h);

				if (h < 100) 
				{
					h = 100 - h;
					y = 110;
					g.FillRectangle(uncommitedBrush, x, y, w, h);
				}

				h = (int)(usage.Available * rate);
				y = 210-70;
				g.FillRectangle(availableBrush, x, y, w, h);

				x = 305 - i * 5 - 5;
				w = 2;
				h = (int)(usage.Pagein);
				y = 210 - h;
				g.FillRectangle(Brushes.LightGray, x, y, w, h);

				x = 308 - i * 5 - 5;
				w = 2;
				h = (int)(usage.Pageout);
				y = 210 - h;
				g.FillRectangle(Brushes.Black, x, y, w, h);

				posu--;
			}
			g.DrawLine(Pens.Black, 5, 210-70, 305, 210-70);
		}
	}
}
